/**
 * Title:       Abstract JMS Topic publisher for ASETS project (ASE Trading System)
 * ClassName:   AbstractTopicPublisher.java
 * Created:     November 21, 2009
 * Author:      Claudiu Vinte
 * Department:  Economic Informatics, The Bucharest Academy of Economic Studies
 *
 * Description: Defines an abstract Topic publisher within OpenMQ broker
 */

package asets.api;

import javax.jms.*;


public abstract class AbstractTopicPublisher
{
    private TopicSession session = null;
    private TopicPublisher publisher = null;
    private TopicConnection connection = null;
    private String userID = null;

    // Constructor used to initialize AbstractTopicPublisher
    public AbstractTopicPublisher(String imqAddress, String topicName, String userID)
            throws JMSException, Exception
    {
        // Create a JMS connection factory
        String addressList = imqAddress;
        com.sun.messaging.TopicConnectionFactory topicConnectionFactory = new com.sun.messaging.TopicConnectionFactory();
        topicConnectionFactory.setProperty(com.sun.messaging.ConnectionConfiguration.imqAddressList, addressList);

        // Create a JMS connection
        TopicConnection topicConnection = topicConnectionFactory.createTopicConnection();
        topicConnection.setClientID(userID);

        // Create JMS session object
        TopicSession topicSession = topicConnection.createTopicSession(false, Session.AUTO_ACKNOWLEDGE);
 
        // Create a JMS topic
        Topic pubTopic = topicSession.createTopic(topicName);

        // Create a JMS publisher
        TopicPublisher topicPublisher = topicSession.createPublisher(pubTopic);

        // Intialize the AbstractTopicPublisher application variables
        this.connection = topicConnection;
        this.session = topicSession;
        this.publisher = topicPublisher;
        this.userID = userID;

        // Start the JMS connection; allows messages to be delivered
        this.connection.start();
    }

    // Close the JMS connection
    public void close() throws JMSException
    {
        if (this.publisher != null)
        {
            this.publisher.close();
            this.publisher = null;
        }        
        if (this.session != null)
        {
            this.session.close();
            this.session = null;
        }
        if (this.connection != null)
        {
            this.connection.close();
            this.connection = null;
        }       
    }

    public TopicSession getSession()
    {
        return this.session;
    }

    public TopicPublisher getPublisher()
    {
        return this.publisher;
    }

    public String getUserID()
    {
        return this.userID;
    }
}
