/**
 * Title:       Abstract JMS Topic subscriber for ASETS project (ASE Trading System)
 * ClassName:   AbstractTopicSubscriber.java
 * Created:     November 21, 2009
 * Author:      Claudiu Vinte
 * Department:  Economic Informatics, The Bucharest Academy of Economic Studies
 *
 * Description: Defines an abstract Topic subscriber within OpenMQ broker
 */

package asets.api;

import javax.jms.*;


public abstract class AbstractTopicSubscriber implements javax.jms.MessageListener
{
    private TopicConnection connection = null;
    private TopicSession session = null;
    private TopicSubscriber subscriber = null;    
    private String userID = null;

    // Constructor used to initialize AbstractTopicSubscriber
    public AbstractTopicSubscriber(String imqAddress, String topicName, String userID)
            throws JMSException, Exception
    {
        // Create a JMS connection factory
        String addressList = imqAddress;
        com.sun.messaging.TopicConnectionFactory topicConnectionFactory = new com.sun.messaging.TopicConnectionFactory();
        topicConnectionFactory.setProperty(com.sun.messaging.ConnectionConfiguration.imqAddressList, addressList);

        // Create a JMS connection
        TopicConnection topicConnection = topicConnectionFactory.createTopicConnection();
        topicConnection.setClientID(userID);

        // Create JMS session object
        TopicSession topicSession = topicConnection.createTopicSession(false, Session.AUTO_ACKNOWLEDGE);

        // Create a JMS topic
        Topic subTopic = topicSession.createTopic(topicName);

        // Create a JMS subscriber
        TopicSubscriber topicSubscriber = topicSession.createSubscriber(subTopic, null, true);

        // Set a JMS message listener
        topicSubscriber.setMessageListener(this);

        // Intialize the AbstractTopicSubscriber application variables
        this.connection = topicConnection;
        this.session = topicSession;
        this.subscriber = topicSubscriber;       
        this.userID = userID;

        // Start the JMS connection; allows messages to be captured
        this.connection.start();
    }

    // Close the JMS connection
    public void close() throws JMSException
    {
       if (this.subscriber != null)
        {
            this.subscriber.close();
            this.subscriber = null;
        }        
        if (this.session!= null)
        {
            this.session.close();
            this.session = null;
        }                
        if (this.connection != null)
        {
            this.connection.close();
            this.connection = null;
        }        
    }

    public TopicSession getSession()
    {
        return this.session;
    }

    public TopicSubscriber getSubsriber()
    {
        return this.subscriber;
    }
    
    public String getUserID()
    {
        return this.userID;
    }
}

