/**
 * Title:       ASETS API (ASE Trading System API)
 * ClassName:   Enums.java
 * Created:     November 14, 2009
 * Author:      Claudiu Vinte
 * Department:  Economic Informatics, The Bucharest Academy of Economic Studies
 *
 * Description: Defines enumerates for ASETS project
 */

package asets.api;


public class Enums implements java.io.Serializable
{
    private static final long serialVersionUID = 1L;

    private Enums()
    {
        // Prevents instantiation
    };

    public static enum Side
    {
        BUY,
        SELL
    };

    public static enum OrderType
    {
        LIMIT_PRICE,
        MARKET_PRICE
    };

    public static enum OrderStatus
    {
        ADD,
        // ADDING,
        ADDED,
        ADD_FAILED,

        UPDATE,
        // UPDATING,
        UPDATED,
        UPDATE_FAILED,

        CANCEL,
        // CANCELING,
        CANCELED,
        CANCEL_FAILED,

        EXECUTED
    };

    public static enum MarketStatus
    {
        PENDING,            // if there has no been any asynchronous message received on CLIENT_ORDER_TOPIC
        ON_MARKET,          // when there has been received a message on CLIENT_ORDER_TOPIC
        PARTIALLY_EXECUTED, // orderQuantity and remainedQuantity to be taken into account for this status
        FULLY_EXECUTED,     // when remainedQuantity is zero
        CLOSED              // when the order is canceled
    };

    public static enum UserStatus
    {
        DISABLED,
        ENABLED,
        PENDING
    };

    public static enum ConnectionStatus
    {
        LOGGED_OUT,
        LOGGED_IN,
        WRONG_CREDENTIALS,
        NOT_REGISTERED,
        PENDING
    };

    public static enum MessageType
    {
        ORDER,
        EXECUTION,
        CONTROL,
        USER,
        PORTFOLIO,
        PRICE,
        ACCOUNT,
        BVB_TRADE,
        
        ORDER_LIST,
        EXECUTION_LIST,
        PORTFOLIO_LIST,
        TRADE_LIST,
        INSTRUMENT_LIST,
        CARD_LIST,
        PRICE_LIST,
        PRICE_RANGE,
        BVB_TRADE_RANGE
    };

    public static enum ControlType
    {
        CLIENT_ORDER_LIST_REQUEST,
        INSTRUMENT_ORDER_LIST_REQUEST,
        MARKET_ORDER_LIST_REQUEST,

        CLIENT_EXECUTION_LIST_REQUEST,
        INSTRUMENT_EXECUTION_LIST_REQUEST,
        MARKET_EXECUTION_LIST_REQUEST,

        PORTFOLIO_LIST_REQUEST,
        TRADE_LIST_REQUEST,
        INSTRUMENT_LIST_REQUEST,
  
        LOG_IN_REQUEST,
        // LOG_IN_REPLY,

        LOG_OUT_REQUEST,
        // LOG_OUT_REPLY,

        KEEP_ALIVE_REQUEST,
        KEEP_ALIVE_REPLY,

        CARD_LIST_REQUEST,

        USER_REQUEST,
        USER_ADD_REQUEST,
        USER_UPDATE_REQUEST,

        ACCOUNT_REQUEST,

        PRICE_REQUEST,
        PRICE_LIST_REQUEST,
        PRICE_RANGE_REQUEST,
        
        BVB_TRADE_RANGE_REQUEST
    };

    public static enum ControlSubType
    {
        // IF NEEDED
    };

    public static enum QueueName
    {
        ORDER_REQUEST_QUEUE,
        ORDER_REPLY_QUEUE,

        LOG_IN_REQUEST_QUEUE,
        LOG_IN_REPLY_QUEUE,

        LOG_OUT_REQUEST_QUEUE,
        LOG_OUT_REPLY_QUEUE,

        // KEEP_ALIVE_REQUEST_QUEUE,
        // KEEP_ALIVE_REPLY_QUEUE,

        CLIENT_ORDER_LIST_REQUEST_QUEUE,
        CLIENT_ORDER_LIST_REPLY_QUEUE,

        CLIENT_EXECUTION_LIST_REQUEST_QUEUE,
        CLIENT_EXECUTION_LIST_REPLY_QUEUE,

        INSTRUMENT_ORDER_LIST_REQUEST_QUEUE,
        INSTRUMENT_ORDER_LIST_REPLY_QUEUE,

        INSTRUMENT_EXECUTION_LIST_REQUEST_QUEUE,
        INSTRUMENT_EXECUTION_LIST_REPLY_QUEUE,
        
        PORTFOLIO_LIST_REQUEST_QUEUE,
        POTFOLIO_LIST_REPLY_QUEUE,

        TRADE_LIST_REQUEST_QUEUE,
        TRADE_LIST_REPLY_QUEUE,

        INSTRUMENT_LIST_REQUEST_QUEUE,
        INSTRUMENT_LIST_REPLY_QUEUE,

        CLIENT_ORDER_QUEUE,
        MARKET_ORDER_QUEUE,
        PROG_ORDER_QUEUE,

        MARKET_EXECUTION_QUEUE,

        CARD_LIST_REQUEST_QUEUE,
        CARD_LIST_REPLY_QUEUE,

        USER_REQUEST_QUEUE,
        USER_REPLY_QUEUE,

        PRICE_REQUEST_QUEUE,
        PRICE_REPLY_QUEUE,

        PRICE_LIST_REQUEST_QUEUE,
        PRICE_LIST_REPLY_QUEUE,
        
        BVB_INSTRUMENT_LIST_REQUEST_QUEUE,
        BVB_INSTRUMENT_LIST_REPLY_QUEUE,
        
        BVB_PRICE_REQUEST_QUEUE,
        BVB_PRICE_REPLY_QUEUE,

        BVB_PRICE_LIST_REQUEST_QUEUE,
        BVB_PRICE_LIST_REPLY_QUEUE,
        
        BVB_PRICE_RANGE_REQUEST_QUEUE,
        BVB_PRICE_RANGE_REPLY_QUEUE,
        
        BVB_TRADE_RANGE_REQUEST_QUEUE,
        BVB_TRADE_RANGE_REPLY_QUEUE
    };

    public static enum TopicName
    {
        CLIENT_ORDER_TOPIC,

        CLIENT_EXECUTION_TOPIC,

        PRICE_TOPIC,
        
        BVB_TRADE_TOPIC
    };
}