/**
 * Title:       Execution topic subscriber for ASETS project (ASE Trading System)
 * ClassName:   ExecutionSubscriber.java
 * Created:     November 22, 2009
 * Author:      Claudiu Vinte
 * Department:  Economic Informatics, The Bucharest Academy of Economic Studies
 *
 * Description: Implements an Execution topic subscriber within OpenMQ broker, by extending AbstractTopicSubscriber class
 */

package asets.api;

import javax.jms.*;


public class ExecutionSubscriber extends AbstractTopicSubscriber
{
    public ExecutionSubscriber(String imqAddress, String topicName, String userID)
            throws JMSException, Exception
    {
        super(imqAddress, topicName, userID);
    }

    // Receive messages from Execution topic
    public void onMessage(Message message)
    {
        try
        {
            if (message instanceof ObjectMessage)
            {
                if ( (message.getStringProperty("MSG_TYPE").equals(Enums.MessageType.EXECUTION.toString())))
                {
                    ObjectMessage objectMessage = (ObjectMessage)message;
                    Execution execution = (Execution)objectMessage.getObject();
                    
                    // Any disired additional filtering should be added in the derived class
                    
                    processExecution(execution);
                }
            }
        } catch (JMSException e) {
            e.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    protected void processExecution(Execution execution) throws Exception
    {

        // TO DO in the derived class

    }
}