/**
 * Title:       Control queue receiver with User reply mechanism for ASETS project (ASE Trading System)
 * ClassName:   KeepAliveReceiverWithReply.java
 * Created:     November 25, 2009
 * Author:      Claudiu Vinte
 * Department:  Economic Informatics, The Bucharest Academy of Economic Studies
 *
 * Description: Implements an Control queue receiver with reply mechanism, within OpenMQ broker by extending AbstractQueueReceiver class
 */

package asets.api;

import javax.jms.*;


public class KeepAliveReceiverWithReply extends AbstractQueueReceiver
{
    public KeepAliveReceiverWithReply(String imqAddress, String queueName, String userID)
            throws JMSException, Exception
    {
        super(imqAddress, queueName, userID);
    }

    // Receive messages from Control queue
    public void onMessage(Message message)
    {
        try
        {
            if (message instanceof ObjectMessage)
            {
                if (message.getStringProperty("MSG_TYPE").equals(Enums.MessageType.CONTROL.toString()) &&
                        message.getStringProperty("CTRL_TYPE").equals(Enums.ControlType.KEEP_ALIVE_REQUEST.toString()))
                {
                    ObjectMessage objectMessage = (ObjectMessage)message;

                    // Process Control request and suply Control reply
                    Control replyControl = processKeepAliveRequest((Control)objectMessage.getObject());

                    // Construct the reply message as an ObjectMessage (Control)
                    ObjectMessage replyMessage = this.getSession().createObjectMessage();
                    replyMessage.setObject(replyControl);

                    replyMessage.setStringProperty("MSG_TYPE", Enums.MessageType.CONTROL.toString());
                    replyMessage.setStringProperty("CTRL_TYPE", Enums.ControlType.KEEP_ALIVE_REPLY.toString());
                    replyMessage.setStringProperty("USER_ID", message.getStringProperty("USER_ID"));

                    replyMessage.setJMSCorrelationID(message.getJMSMessageID());

                    // Create the sender and send Control
                    QueueSender replySender = this.getSession().createSender((Queue)message.getJMSReplyTo());
                    replySender.setTimeToLive(30000);     // The reply will be valid for 30 sec.
                    replySender.send(replyMessage);
                    
                    replySender.close();
                }
            }
        } catch (JMSException e) {
            e.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    protected Control processKeepAliveRequest(Control control) throws Exception
    {
        Control replyControl = new Control();

        // TO DO in the derived class
        // control.setControlType(Enums.ControlType.KEEP_ALIVE_REPLY);

        return replyControl;
    }
}
