/**
 * Title:       Control queue receiver with User reply mechanism for ASETS project (ASE Trading System)
 * ClassName:   LogInReceiverWithReply.java
 * Created:     November 25, 2009
 * Author:      Claudiu Vinte
 * Department:  Economic Informatics, The Bucharest Academy of Economic Studies
 *
 * Description: Implements an Control queue receiver with reply mechanism, within OpenMQ broker by extending AbstractQueueReceiver class
 */

package asets.api;

import javax.jms.*;


public class LogInReceiverWithReply extends AbstractQueueReceiver
{
    public LogInReceiverWithReply(String imqAddress, String queueName, String userID)
            throws JMSException, Exception
    {
        super(imqAddress, queueName, userID);
    }

    // Receive messages from Control queue
    public void onMessage(Message message)
    {
        try
        {
            if (message instanceof ObjectMessage)
            {
                if (message.getStringProperty("MSG_TYPE").equals(Enums.MessageType.CONTROL.toString()) &&
                        message.getStringProperty("CTRL_TYPE").equals(Enums.ControlType.LOG_IN_REQUEST.toString()))
                {
                    ObjectMessage objectMessage = (ObjectMessage)message;

                    // Process Control request and suply User reply
                    User replyUser = processLogInRequest((Control)objectMessage.getObject());

                    // Construct the reply message as an ObjectMessage (User)
                    ObjectMessage replyMessage = this.getSession().createObjectMessage();
                    replyMessage.setObject(replyUser);

                    replyMessage.setStringProperty("MSG_TYPE", Enums.MessageType.USER.toString());
                    replyMessage.setStringProperty("USER_ID", message.getStringProperty("USER_ID"));
                    
                    replyMessage.setJMSCorrelationID(message.getJMSMessageID());

                    // Create the sender and send User
                    QueueSender replySender = this.getSession().createSender((Queue)message.getJMSReplyTo());
                    replySender.setTimeToLive(30000);     // The reply will be valid for 30 sec.
                    replySender.send(replyMessage);
                    
                    replySender.close();
                }
            }
        } catch (JMSException e) {
            e.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    protected User processLogInRequest(Control control) throws Exception
    {
        User replyUser = new User();

        // TO DO in the derived class

        return replyUser;
    }
}
