/**
 * Title:       Order queue receiver for ASETS project (ASE Trading System)
 * ClassName:   OrderReceiverNoReply.java
 * Created:     November 22, 2009
 * Author:      Claudiu Vinte
 * Department:  Economic Informatics, The Bucharest Academy of Economic Studies
 *
 * Description: Implements an Order queue receiver within OpenMQ broker by extending AbstractQueueReceiver class
 */

package asets.api;

import javax.jms.*;


public class OrderReceiverNoReply extends AbstractQueueReceiver
{
    public OrderReceiverNoReply(String imqAddress, String queueName, String userID)
            throws JMSException, Exception
    {
        super(imqAddress, queueName, userID);
    }

    // Receive messages from Order queue
    public void onMessage(Message message)
    {
        try
        {
            if (message instanceof ObjectMessage)
            {   
                if (message.getStringProperty("MSG_TYPE").equals(Enums.MessageType.ORDER.toString()))
                // Client Orders are received by ESE either from OMS or from PROG
                // Market Orders are received by OMS from ESE
                // ESE      - Exchange Simulation Engine
                // OMS      - Order Management Server
                // PROG     - Pseudo Random Order Generator

                // TO DO in the derived class
 
                {
                    ObjectMessage objectMessage = (ObjectMessage)message;
                    processOrder((Order)objectMessage.getObject());
                }
            }
        } catch (JMSException e) {
            e.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    protected void processOrder(Order order) throws Exception
    {

        // TO DO in the derived class

    }
}
