/**
 * Title:       Order queue receiver with reply mechanism for ASETS project (ASE Trading System)
 * ClassName:   OrderReceiverWithReply.java
 * Created:     November 22, 2009
 * Author:      Claudiu Vinte
 * Department:  Economic Informatics, The Bucharest Academy of Economic Studies
 *
 * Description: Implements an Order queue receiver with reply mechanism, within OpenMQ broker by extending AbstractQueueReceiver class
 */

package asets.api;

import javax.jms.*;


public class OrderReceiverWithReply extends AbstractQueueReceiver
{
    public OrderReceiverWithReply(String imqAddress, String queueName, String userID)
            throws JMSException, Exception
    {
        super(imqAddress, queueName, userID);
    }

    // Receive messages from Order queue
    public void onMessage(Message message)
    {
        try
        {
            if (message instanceof ObjectMessage)
            {
                if (message.getStringProperty("MSG_TYPE").equals(Enums.MessageType.ORDER.toString()))
                {
                    ObjectMessage objectMessage = (ObjectMessage)message;             
			
                    // Process Order request and suply Order reply
                    Order replyOrder = processOrderRequest((Order)objectMessage.getObject());

                    if( replyOrder == null ) return;

                    // Construct the reply message as an ObjectMessage (Order)
                    ObjectMessage replyMessage = this.getSession().createObjectMessage();
                    replyMessage.setObject(replyOrder);

                    replyMessage.setStringProperty("MSG_TYPE", Enums.MessageType.ORDER.toString());
                    replyMessage.setStringProperty("USER_ID", message.getStringProperty("USER_ID"));
                    
                    replyMessage.setJMSCorrelationID(message.getJMSMessageID());
                    
                    // Create the sender and send Order
                    QueueSender replySender = this.getSession().createSender((Queue)message.getJMSReplyTo());
                    replySender.setTimeToLive(30000);     // The reply will be valid for 30 sec.
                    replySender.send(replyMessage);
                    
                    replySender.close();
                }
            }
        } catch (JMSException e) {
            e.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    protected Order processOrderRequest(Order order) throws Exception
    {
        Order replyOrder = new Order();

        // TO DO in the derived class

        return replyOrder;
    }
}
