/**
 * Title:       Order queue sender for ASETS project (ASE Trading System)
 * ClassName:   OrderSenderNoReply.java
 * Created:     November 22, 2009
 * Author:      Claudiu Vinte
 * Department:  Economic Informatics, The Bucharest Academy of Economic Studies
 *
 * Description: Implements an Order queue sender, within OpenMQ broker, by extending AbstractQueueSenderNoReply class
 */

package asets.api;

import javax.jms.*;


public class OrderSenderNoReply extends AbstractQueueSenderNoReply
{
    public OrderSenderNoReply(String imqAddress, String queueName, String userID)
            throws JMSException, Exception
    {
        super(imqAddress, queueName, userID);
    }

    // Create and send an Order message
    public void sendOrder(Order order) throws JMSException
    {
        // Construct the message as an ObjectMessage (Order)
        ObjectMessage message = this.getSession().createObjectMessage();
        message.setObject(order);

        message.setStringProperty("MSG_TYPE", Enums.MessageType.ORDER.toString());
        message.setStringProperty("USER_ID", this.getUserID());

        // Client Orders are received by ESE either from OMS or from PROG
        // Market Orders are received by OMS from ESE
        // ESE      - Exchange Simulation Engine
        // OMS      - Order Management Server
        // PROG     - Pseudo Random Order Generator
  

        // TO DO in the derived class

        this.getSender().send(message);
    }
}

