/**
 * Title:       Order topic subscriber for ASETS project (ASE Trading System)
 * ClassName:   OrderSubscriber.java
 * Created:     November 22, 2009
 * Author:      Claudiu Vinte
 * Department:  Economic Informatics, The Bucharest Academy of Economic Studies
 *
 * Description: Implements an Order topic subscriber within OpenMQ broker, by extending AbstractTopicSubscriber class
 */

package asets.api;

import javax.jms.*;


public class OrderSubscriber extends AbstractTopicSubscriber
{
    public OrderSubscriber(String imqAddress, String topicName, String userID)
            throws JMSException, Exception
    {
        super(imqAddress, topicName, userID);
    }

    // Receive messages from Order topic
    public void onMessage(Message message)
    {
        try
        {
            if (message instanceof ObjectMessage)
            {
                if ( (message.getStringProperty("MSG_TYPE").equals(Enums.MessageType.ORDER.toString())))
                {
                    ObjectMessage objectMessage = (ObjectMessage)message;
                    Order order = (Order)objectMessage.getObject();
                    
                    // Any disired additional filtering should be added in the derived class
                  
                    processOrder(order);
                }
            }
        } catch (JMSException e) {
            e.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    protected void processOrder(Order order) throws Exception
    {

        // TO DO in the derived class

    }
}