/**
 * Title:       Control queue receiver with PriceList reply mechanism for ASETS project (ASE Trading System)
 * ClassName:   PriceListReceiverWithReply.java
 * Created:     July 21, 2011
 * Author:      Claudiu Vinte
 * Department:  Economic Informatics, The Bucharest Academy of Economic Studies
 *
 * Description: Implements an Control queue receiver with reply mechanism, within OpenMQ broker by extending AbstractQueueReceiver class
 */

package asets.api;

import javax.jms.*;


public class PriceListReceiverWithReply extends AbstractQueueReceiver
{
    public PriceListReceiverWithReply(String imqAddress, String queueName, String userID)
            throws JMSException, Exception
    {
        super(imqAddress, queueName, userID);
    }

    // Receive messages from Control queue
    public void onMessage(Message message)
    {
        try
        {
            if (message instanceof ObjectMessage)
            {
                if (message.getStringProperty("MSG_TYPE").equals(Enums.MessageType.CONTROL.toString()) &&
                        message.getStringProperty("CTRL_TYPE").equals(Enums.ControlType.PRICE_LIST_REQUEST.toString()))
                {
                    ObjectMessage objectMessage = (ObjectMessage)message;

                    // Process Control request and suply Price reply
                    PriceList replyPriceList = processPriceListRequest((Control)objectMessage.getObject());

                    // Construct the reply message as an ObjectMessage (Price)
                    ObjectMessage replyMessage = this.getSession().createObjectMessage();
                    replyMessage.setObject(replyPriceList);

                    replyMessage.setStringProperty("MSG_TYPE", Enums.MessageType.PRICE_LIST.toString());
                    replyMessage.setStringProperty("USER_ID", message.getStringProperty("USER_ID"));

                    replyMessage.setJMSCorrelationID(message.getJMSMessageID());

                    // Create the sender and send Price
                    QueueSender replySender = this.getSession().createSender((Queue)message.getJMSReplyTo());
                    replySender.setTimeToLive(30000);     // The reply will be valid for 30 sec.
                    replySender.send(replyMessage);
                    
                    replySender.close();
                }
            }
        } catch (JMSException e) {
            e.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    protected PriceList processPriceListRequest(Control control) throws Exception
    {
        PriceList replyPriceList = new PriceList();

        // TO DO in the derived class

        return replyPriceList;
    }
}
