/**
 * Title:       PriceList request sender with reply mechanism for ASETS project (ASE Trading System)
 * ClassName:   PriceListSenderWithReply.java
 * Created:     July 21, 2011
 * Author:      Claudiu Vinte
 * Department:  Economic Informatics, The Bucharest Academy of Economic Studies
 *
 * Description: Implements a Control queue sender, with reply mechanism, within OpenMQ broker, by extending AbstractQueueSenderWithReply class
 */

package asets.api;

import javax.jms.*;


public class PriceListSenderWithReply extends AbstractQueueSenderWithReply
{
    public PriceListSenderWithReply(String imqAddress, String requestQueueName,
            String replyQueueName, String userID) throws JMSException, Exception
    {
        super(imqAddress, requestQueueName, replyQueueName, userID);
    }

    // Create and send a Control message, waiting for PriceList message reply
    public PriceList sendControlGetReply(Control control) throws JMSException, Exception
    {
        // Construct Control request message by supplying
        // the number of days back from today or date, 0 (zero) means the current date
        // control.setControlType(Enums.ControlType.PRICE_LIST_REQUEST);
        // control.setRequestText(Integer.toString(noOfDays));
        // or control.setRequestText(Date.toString(date));       


        ObjectMessage message = this.getSession().createObjectMessage();
        message.setObject(control);

        message.setStringProperty("MSG_TYPE", Enums.MessageType.CONTROL.toString());
        message.setStringProperty("CTRL_TYPE", Enums.ControlType.PRICE_LIST_REQUEST.toString());
        message.setStringProperty("USER_ID", control.getUserID());
        message.setJMSReplyTo(this.getReplyQueue());

        // Create Control sender and send the message
        QueueSender requestSender = this.getSession().createSender(this.getRequestQueue());
        requestSender.setTimeToLive(30000);     // The request will be valid for 30 sec.
        requestSender.send(message);

        // Wait and process PriceList reply
        String filter = "JMSCorrelationID = '" + message.getJMSMessageID() + "'";
        QueueReceiver replyReceiver= this.getSession().createReceiver(this.getReplyQueue(), filter);

        Message replyMessage = replyReceiver.receive(33000);    // Waiting the reply for maximum 33 sec.
        if (replyMessage == null)
        {
            System.out.println("Destination not responding");
        } else {
            if (replyMessage instanceof ObjectMessage)
            {
                if (replyMessage.getStringProperty("MSG_TYPE").equals(Enums.MessageType.PRICE_LIST.toString()) &&
                        replyMessage.getStringProperty("USER_ID").equals(control.getUserID()))
                {
                    ObjectMessage replyObject = (ObjectMessage)replyMessage;
                    PriceList replyPriceList = (PriceList)replyObject.getObject();

                    replyReceiver.close();
                    requestSender.close();
                    
                    return replyPriceList;
                }
            } else {
                throw new IllegalArgumentException("Invalid message type");
            }
        }

        replyReceiver.close();
        requestSender.close();
        
        return null;
    }
}

