/**
 * Title:       ASETS API (ASE Trading System API)
 * ClassName:   BVBTrade.java
 * Created:     March 10, 2012
 * Author:      Claudiu Vinte
 * Department:  Economic Informatics, The Bucharest Academy of Economic Studies
 *
 * Description: Implements BVBTrade class for ASETS project
 */

package asets.api.collateral;


public class BVBTrade implements java.io.Serializable, Comparable<BVBTrade> {
	private static final long serialVersionUID = 1L;

	private long tradeID;
	private java.lang.String symbol;
	private java.lang.String marketCode;
	private java.util.Calendar tradeTime;
	private long volume;
	private float price;
	private float refPrice;

	public BVBTrade() {
		/* empty constructor */
	}

	public BVBTrade(long tradeID, java.lang.String symbolcode,
			java.lang.String marketcode, java.util.Calendar tradeTime,
			long volume, float price, float refPrice) {
		this.tradeID = tradeID;
		this.symbol = symbolcode;
		this.marketCode = marketcode;
		this.tradeTime = tradeTime;
		this.volume = volume;
		this.price = price;
		this.refPrice = refPrice;
	}
        
 	public BVBTrade(BVBTrade bvbTrade) {
		this.tradeID = bvbTrade.getTradeID();
		this.symbol = bvbTrade.getSymbol();
		this.marketCode = bvbTrade.getMarketCode();
		this.tradeTime = bvbTrade.getTradeTime();
		this.volume = bvbTrade.getVolume();
		this.price = bvbTrade.getPrice();
		this.refPrice = bvbTrade.getRefPrice();
	}       

	/**
	 * Gets the tradeID value for this trade.
	 * 
	 * @return tradeID
	 */
	public long getTradeID() {
		return tradeID;
	}

	/**
	 * Sets the tradeID value for this trade.
	 * 
	 * @param tradeID
	 */
	public void setTradeID(long tradeID) {
		this.tradeID = tradeID;
	}

	/**
	 * Gets the symbol code value for this trade.
	 * 
	 * @return symbol code
	 */
	public java.lang.String getSymbol() {
		return symbol;
	}

	/**
	 * Sets the symbol code value for this trade.
	 * 
	 * @param symbolcode
	 */
	public void setSymbol(java.lang.String symbolcode) {
		this.symbol = symbolcode;
	}

	/**
	 * Gets the market code value for this trade.
	 * 
	 * @return market code
	 */
	public java.lang.String getMarketCode() {
		return marketCode;
	}

	/**
	 * Sets the market code value for this trade.
	 * 
	 * @param market code
	 */
	public void setMarketCode(java.lang.String marketcode) {
		this.marketCode = marketcode;
	}

	/**
	 * Gets the tradeTime value for this trade.
	 * 
	 * @return tradeTime
	 */
	public java.util.Calendar getTradeTime() {
		return tradeTime;
	}

	/**
	 * Sets the tradeTime value for this trade.
	 * 
	 * @param tradeTime
	 */
	public void setTradeTime(java.util.Calendar tradeTime) {
		this.tradeTime = tradeTime;
	}

	/**
	 * Gets the volume value for this trade.
	 * 
	 * @return volume
	 */
	public long getVolume() {
		return volume;
	}

	/**
	 * Sets the volume value for this trade.
	 * 
	 * @param volume
	 */
	public void setVolume(long volume) {
		this.volume = volume;
	}

	/**
	 * Gets the price value for this trade.
	 * 
	 * @return price
	 */
	public float getPrice() {
		return price;
	}

	/**
	 * Sets the price value for this trade.
	 * 
	 * @param price
	 */
	public void setPrice(float price) {
		this.price = price;
	}

	/**
	 * @return the refPrice
	 */
	public float getRefPrice() {
		return refPrice;
	}

	/**
	 * @param refPrice the refPrice to set
	 */
	public void setRefPrice(float refPrice) {
		this.refPrice = refPrice;
	}

	@Override
	public int compareTo(BVBTrade theOther) {
		if( theOther == null ) return 1;
		return this.tradeTime.compareTo(theOther.tradeTime);
	}
}
