/**
 * Title:       Control queue receiver with BVBTradeRange reply mechanism for ASETS project (ASE Trading System)
 * ClassName:   BVBTradeRangeReceiverWithReply.java
 * Created:     May 5, 2012
 * Author:      Claudiu Vinte
 * Department:  Economic Informatics, The Bucharest Academy of Economic Studies
 *
 * Description: Implements a Control queue receiver with reply mechanism, within OpenMQ broker by extending AbstractQueueReceiver class
 */

package asets.api.collateral;

import asets.api.AbstractQueueReceiver;
import asets.api.Control;
import asets.api.Enums;
import javax.jms.*;


public class BVBTradeRangeReceiverWithReply extends AbstractQueueReceiver
{
    public BVBTradeRangeReceiverWithReply(String imqAddress, String queueName, String userID)
            throws JMSException, Exception
    {
        super(imqAddress, queueName, userID);
    }

    // Receive messages from Control queue
    public void onMessage(Message message)
    {
        try
        {
            if (message instanceof ObjectMessage)
            {
                if (message.getStringProperty("MSG_TYPE").equals(Enums.MessageType.CONTROL.toString()) &&
                        message.getStringProperty("CTRL_TYPE").equals(Enums.ControlType.BVB_TRADE_RANGE_REQUEST.toString()))
                {
                    ObjectMessage objectMessage = (ObjectMessage)message;

                    // Process Control request and suply BVBTrade reply
                    BVBTradeRange replyBVBTradeRange = processBVBTradeRangeRequest((Control)objectMessage.getObject());

                    // Construct the reply message as an ObjectMessage (BVBTrade)
                    ObjectMessage replyMessage = this.getSession().createObjectMessage();
                    replyMessage.setObject(replyBVBTradeRange);

                    replyMessage.setStringProperty("MSG_TYPE", Enums.MessageType.BVB_TRADE_RANGE.toString());
                    replyMessage.setStringProperty("USER_ID", message.getStringProperty("USER_ID"));

                    replyMessage.setJMSCorrelationID(message.getJMSMessageID());

                    // Create the sender and send BVBTrade
                    QueueSender replySender = this.getSession().createSender((Queue)message.getJMSReplyTo());
                    replySender.setTimeToLive(30000);     // The reply will be valid for 30 sec.
                    replySender.send(replyMessage);
                    
                    replySender.close();
                }
            }
        } catch (JMSException e) {
            e.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    protected BVBTradeRange processBVBTradeRangeRequest(Control control) throws Exception
    {
        BVBTradeRange replyBVBTradeRange = new BVBTradeRange();

        // TO DO in the derived class

        return replyBVBTradeRange;
    }
}
