/**
 * Title:       BVBTradeRange request sender with reply mechanism for ASETS project (ASE Trading System)
 * ClassName:   BVBTradeRangeSenderWithReply.java
 * Created:     May 5, 2012
 * Author:      Claudiu Vinte
 * Department:  Economic Informatics, The Bucharest Academy of Economic Studies
 *
 * Description: Implements a Control queue sender, with reply mechanism, within OpenMQ broker, by extending AbstractQueueSenderWithReply class
 */

package asets.api.collateral;

import asets.api.AbstractQueueSenderWithReply;
import asets.api.Control;
import asets.api.Enums;
import javax.jms.*;


public class BVBTradeRangeSenderWithReply extends AbstractQueueSenderWithReply
{
    public BVBTradeRangeSenderWithReply(String imqAddress, String requestQueueName,
            String replyQueueName, String userID) throws JMSException, Exception
    {
        super(imqAddress, requestQueueName, replyQueueName, userID);
    }

    // Create and send a Control message, waiting for ControlRange message reply
    public BVBTradeRange sendControlGetReply(Control control) throws JMSException, Exception
    {
        // Construct Control request message by supplying
        // the number of days back from today or date, 0 (zero) means the current date
        // control.setControlType(Enums.ControlType.BVB_TRADE_RANGE_REQUEST);
        // control.setRequestText(symbol);
        // control.setControl1(Integer.toString(noOfDays));
        //  or control.setControl1(Date.toString(date));       

        ObjectMessage message = this.getSession().createObjectMessage();
        message.setObject(control);

        message.setStringProperty("MSG_TYPE", Enums.MessageType.CONTROL.toString());
        message.setStringProperty("CTRL_TYPE",control.getControlType().toString());
        message.setStringProperty("USER_ID", control.getUserID());
        message.setJMSReplyTo(this.getReplyQueue());

        // Create Control sender and send the message
        QueueSender requestSender = this.getSession().createSender(this.getRequestQueue());
        requestSender.setTimeToLive(30000);     // The request will be valid for 30 sec.
        requestSender.send(message);

        // Wait and process BVBTradeRange reply
        String filter = "JMSCorrelationID = '" + message.getJMSMessageID() + "'";
        QueueReceiver replyReceiver= this.getSession().createReceiver(this.getReplyQueue(), filter);

        Message replyMessage = replyReceiver.receive(33000);    // Waiting the reply for maximum 33 sec.
        if (replyMessage == null)
        {
            System.out.println("Destination not responding");
        } else {
            if (replyMessage instanceof ObjectMessage)
            {
                if (replyMessage.getStringProperty("MSG_TYPE").equals(Enums.MessageType.BVB_TRADE_RANGE.toString()) &&
                        replyMessage.getStringProperty("USER_ID").equals(control.getUserID()))
                {
                    ObjectMessage replyObject = (ObjectMessage)replyMessage;
                    BVBTradeRange replyBVBTradeRange = (BVBTradeRange)replyObject.getObject();

                    replyReceiver.close();
                    requestSender.close();
                    
                    return replyBVBTradeRange;
                }
            } else {
                throw new IllegalArgumentException("Invalid message type");
            }
        }

        replyReceiver.close();
        requestSender.close();
        
        return null;
    }
}
