/**
 * Title:       RectangleList request sender with reply mechanism for ASETS project (ASE Trading System)
 * ClassName:   RectangleListSenderWithReply.java
 * Created:     March 10, 2010
 * Author:      Claudiu Vinte
 * Department:  Economic Informatics, The Bucharest Academy of Economic Studies
 *
 * Description: Implements a Control queue sender, with reply mechanism, within OpenMQ broker, by extending AbstractQueueSenderWithReply class
 */

package asets.api.collateral;

import javax.jms.*;
import asets.api.*;
// import card.engine.CARDLayout;


public class RectangleListSenderWithReply extends AbstractQueueSenderWithReply
{
    public RectangleListSenderWithReply(String imqAddress, String requestQueueName,
            String replyQueueName, String userID) throws JMSException, Exception
    {
        super(imqAddress, requestQueueName, replyQueueName, userID);
    }

    // Create and send a Control message, waiting for ControlList message reply
    public RectangleList sendControlGetReply(Control control) throws JMSException, Exception
    {
        // Construct Control request message
        // control.setControlType(Enums.ControlType.CARD_LIST_REQUEST);
        // control.setRequestText(CARDLayout.FormFactor.ONE_THIRD.toString());
        // or control.setRequestText(CARDLayout.FormFactor.HALF.toString());
        // or control.setRequestText(CARDLayout.FormFactor.TWO_THIRD.toString());

        ObjectMessage message = this.getSession().createObjectMessage();
        message.setObject(control);

        message.setStringProperty("MSG_TYPE", Enums.MessageType.CONTROL.toString());
        message.setStringProperty("CTRL_TYPE", Enums.ControlType.CARD_LIST_REQUEST.toString());
        message.setStringProperty("USER_ID", control.getUserID());
        message.setJMSReplyTo(this.getReplyQueue());

        // Create Control sender and send the message
        QueueSender requestSender = this.getSession().createSender(this.getRequestQueue());
        requestSender.setTimeToLive(30000);     // The request will be valid for 30 sec.
        requestSender.send(message);

        // Wait and process RectangleList reply
        String filter = "JMSCorrelationID = '" + message.getJMSMessageID() + "'";
        QueueReceiver replyReceiver= this.getSession().createReceiver(this.getReplyQueue(), filter);

        Message replyMessage = replyReceiver.receive(33000);    // Waiting the reply for maximum 33 sec.
        if (replyMessage == null)
        {
            System.out.println("Destination not responding");
        } else {
            if (replyMessage instanceof ObjectMessage)
            {
                if (replyMessage.getStringProperty("MSG_TYPE").equals(Enums.MessageType.CARD_LIST.toString()) &&
                        replyMessage.getStringProperty("USER_ID").equals(control.getUserID()))
                {
                    ObjectMessage replyObject = (ObjectMessage)replyMessage;
                    RectangleList replyCARDRectangleList = (RectangleList)replyObject.getObject();

                    replyReceiver.close();
                    requestSender.close();
                    
                    return replyCARDRectangleList;
                }
            } else {
                throw new IllegalArgumentException("Invalid message type");
            }
        }

        replyReceiver.close();
        requestSender.close();
                    
        return null;
    }
}

